#!/usr/bin/env python3
# Copyright 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import datetime
import sys

import pbr.version
import subunit
from subunit import iso8601


__version__ = pbr.version.VersionInfo('os_testr').version_string()


def main():
    if '--version' in sys.argv:
        print(__version__)
        exit(0)

    start_time = datetime.datetime.fromtimestamp(float(sys.argv[1])).replace(
        tzinfo=iso8601.UTC)
    elapsed_time = datetime.timedelta(seconds=int(sys.argv[2]))
    stop_time = start_time + elapsed_time

    if len(sys.argv) > 3:
        status = sys.argv[3]
    else:
        status = 'success'

    if len(sys.argv) > 4:
        test_id = sys.argv[4]
    else:
        test_id = 'devstack'

    # Write the subunit test
    output = subunit.v2.StreamResultToBytes(sys.stdout)
    output.startTestRun()
    output.status(timestamp=start_time, test_id=test_id)
    # Write the end of the test
    output.status(test_status=status, timestamp=stop_time, test_id=test_id)
    output.stopTestRun()


if __name__ == '__main__':
    main()



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# http://www.sphinx-doc.org/en/master/config

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
# import os
# import sys
# sys.path.insert(0, os.path.abspath('.'))


# -- Project information -----------------------------------------------------

project = 'ostestr Release Notes'
copyright = '2019, ostestr Developers'
author = 'ostestr Developers'


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'openstackdocstheme',
    'reno.sphinxext',
]

# openstackdocstheme options
openstackdocs_repo_name = 'openstack/os-testr'
openstackdocs_auto_name = False
openstackdocs_bug_project = 'os-testr'
openstackdocs_bug_tag = ''

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The encoding of source files.
# source_encoding = 'utf-8-sig'

# The master toctree document.
master_doc = 'index'

# Release do not need a version number in the title, they
# cover multiple versions.
# The full version, including alpha/beta/rc tags.
release = ''
# The short X.Y version.
version = ''

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'native'

# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'openstackdocs'

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
# html_static_path = ['_static']

# Output file base name for HTML help builder.
htmlhelp_basename = 'ostestrReleaseNotesdoc'



# Copyright 2016 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
import os
import subprocess


def _get_test_list(regex, env=None):
    env = env or copy.deepcopy(os.environ)
    testr_args = ['stestr', 'list']
    if regex:
        testr_args.append(regex)
    proc = subprocess.Popen(testr_args, env=env,
                            stdout=subprocess.PIPE, universal_newlines=True)
    out = proc.communicate()[0]
    raw_test_list = out.split('\n')
    bad = False
    test_list = []
    exclude_list = ['OS_', 'CAPTURE', 'TEST_TIMEOUT', 'PYTHON',
                    'subunit.run discover']
    for line in raw_test_list:
        for exclude in exclude_list:
            if exclude in line or not line:
                bad = True
                break
        if not bad:
            test_list.append(line)
        bad = False
    return test_list


def print_skips(regex, message):
    test_list = _get_test_list(regex)
    if test_list:
        if message:
            print(message)
        else:
            print('Skipped because of regex %s:' % regex)
        for test in test_list:
            print(test)
        # Extra whitespace to separate
        print('\n')


def path_to_regex(path):
    root, _ = os.path.splitext(path)
    return root.replace('/', '.')


def get_regex_from_whitelist_file(file_path):
    lines = []
    with open(file_path) as white_file:
        for line in white_file.read().splitlines():
            split_line = line.strip().split('#')
            # Before the # is the regex
            line_regex = split_line[0].strip()
            if line_regex:
                lines.append(line_regex)
    return '|'.join(lines)


def get_regex_from_blacklist_file(file_path, print_exclude=False):
    exclude_regex = ''
    with open(file_path, 'r') as black_file:
        exclude_regex = ''
        for line in black_file:
            raw_line = line.strip()
            split_line = raw_line.split('#')
            # Before the # is the regex
            line_regex = split_line[0].strip()
            if len(split_line) > 1:
                # After the # is a comment
                comment = split_line[1].strip()
            else:
                comment = ''
            if line_regex:
                if print_exclude:
                    print_skips(line_regex, comment)
                if exclude_regex:
                    exclude_regex = '|'.join([line_regex, exclude_regex])
                else:
                    exclude_regex = line_regex
        if exclude_regex:
            exclude_regex = "(?!" + exclude_regex + ")"
    return exclude_regex


def construct_regex(blacklist_file, whitelist_file, regex, print_exclude):
    """Deprecated, please use testlist_builder.construct_list instead."""
    bregex = ''
    wregex = ''
    pregex = ''

    if blacklist_file:
        bregex = get_regex_from_blacklist_file(blacklist_file, print_exclude)
    if whitelist_file:
        wregex = get_regex_from_whitelist_file(whitelist_file)
    if regex:
        pregex = regex
    combined_regex = '^%s.*(%s).*$' % (bregex, '|'.join(
        filter(None, [pregex, wregex])
    ))

    return combined_regex



# Copyright 2016 RedHat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from os_testr import regex_builder
import re


def black_reader(blacklist_file):
    black_file = open(blacklist_file, 'r')
    regex_comment_lst = []  # tupple of (regex_compild, msg, skipped_lst)
    for line in black_file:
        raw_line = line.strip()
        split_line = raw_line.split('#')
        # Before the # is the regex
        line_regex = split_line[0].strip()
        if len(split_line) > 1:
            # After the # is a comment
            comment = ''.join(split_line[1:]).strip()
        else:
            comment = 'Skipped because of regex %s:' % line_regex
        if not line_regex:
            continue
        regex_comment_lst.append((re.compile(line_regex), comment, []))
    return regex_comment_lst


def print_skips(regex, message, test_list):
    for test in test_list:
        print(test)
    # Extra whitespace to separate
    print('\n')


def construct_list(blacklist_file, whitelist_file, regex, black_regex,
                   print_exclude):
    """Filters the discovered test cases

    :retrun: iterable of strings. The strings are full
        test cases names, including tags like.:
        "project.api.TestClass.test_case[positive]"
    """

    if not regex:
        regex = ''  # handle the other false things

    if whitelist_file:
        white_re = regex_builder.get_regex_from_whitelist_file(whitelist_file)
    else:
        white_re = ''

    if not regex and white_re:
        regex = white_re
    elif regex and white_re:
        regex = '|'.join((regex, white_re))

    if blacklist_file:
        black_data = black_reader(blacklist_file)
    else:
        black_data = None

    if black_regex:
        msg = "Skipped because of regex provided as a command line argument:"
        record = (re.compile(black_regex), msg, [])
        if black_data:
            black_data.append(record)
        else:
            black_data = [record]

    search_filter = re.compile(regex)

    # NOTE(afazekas): we do not want to pass a giant re
    # to an external application due to the arg length limitatios
    list_of_test_cases = [test_case for test_case in
                          regex_builder._get_test_list('')
                          if search_filter.search(test_case)]
    set_of_test_cases = set(list_of_test_cases)

    if not black_data:
        return set_of_test_cases

    # NOTE(afazekas): We might use a faster logic when the
    # print option is not requested
    for (rex, msg, s_list) in black_data:
        for test_case in list_of_test_cases:
            if rex.search(test_case):
                # NOTE(mtreinish): In the case of overlapping regex the test
                # case might have already been removed from the set of tests
                if test_case in set_of_test_cases:
                    set_of_test_cases.remove(test_case)
                    s_list.append(test_case)

    if print_exclude:
        for (rex, msg, s_list) in black_data:
            if s_list:
                print_skips(rex, msg, s_list)
    return set_of_test_cases



# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import io
from unittest import mock

from os_testr import regex_builder as os_testr
from os_testr.tests import base


class TestPathToRegex(base.TestCase):

    def test_file_name(self):
        result = os_testr.path_to_regex("tests/network/v2/test_net.py")
        self.assertEqual("tests.network.v2.test_net", result)
        result = os_testr.path_to_regex("openstack/tests/network/v2")
        self.assertEqual("openstack.tests.network.v2", result)


class TestConstructRegex(base.TestCase):
    def test_regex_passthrough(self):
        result = os_testr.construct_regex(None, None, 'fake_regex', False)
        self.assertEqual(result, '^.*(fake_regex).*$')

    def test_blacklist_regex_with_comments(self):
        with io.StringIO() as blacklist_file:
            for i in range(4):
                blacklist_file.write(u'fake_regex_%s # A Comment\n' % i)
            blacklist_file.seek(0)
            with mock.patch('six.moves.builtins.open',
                            return_value=blacklist_file):
                result = os_testr.construct_regex(
                    'fake_path', None, None, False)
            self.assertEqual(result, "^(?!fake_regex_3|fake_regex_2|"
                                     "fake_regex_1|fake_regex_0).*().*$")

    def test_whitelist_regex_with_comments(self):
        with io.StringIO() as whitelist_file:
            for i in range(4):
                whitelist_file.write(u'fake_regex_%s # A Comment\n' % i)
            whitelist_file.seek(0)
            with mock.patch('six.moves.builtins.open',
                            return_value=whitelist_file):
                result = os_testr.construct_regex(
                    None, 'fake_path', None, False)
            self.assertEqual(
                result,
                "^.*(fake_regex_0|fake_regex_1|fake_regex_2|fake_regex_3).*$")

    def test_blacklist_regex_without_comments(self):
        with io.StringIO() as blacklist_file:
            for i in range(4):
                blacklist_file.write(u'fake_regex_%s\n' % i)
            blacklist_file.seek(0)
            with mock.patch('six.moves.builtins.open',
                            return_value=blacklist_file):
                result = os_testr.construct_regex(
                    'fake_path', None, None, False)
            self.assertEqual(result, "^(?!fake_regex_3|fake_regex_2|"
                                     "fake_regex_1|fake_regex_0).*().*$")

    def test_blacklist_regex_with_comments_and_regex(self):
        with io.StringIO() as blacklist_file:
            for i in range(4):
                blacklist_file.write(u'fake_regex_%s # Comments\n' % i)
            blacklist_file.seek(0)
            with mock.patch('six.moves.builtins.open',
                            return_value=blacklist_file):
                result = os_testr.construct_regex('fake_path', None,
                                                  'fake_regex', False)

                expected_regex = (
                    "^(?!fake_regex_3|fake_regex_2|fake_regex_1|"
                    "fake_regex_0).*(fake_regex).*$")
                self.assertEqual(result, expected_regex)

    def test_blacklist_regex_without_comments_and_regex(self):
        with io.StringIO() as blacklist_file:
            for i in range(4):
                blacklist_file.write(u'fake_regex_%s\n' % i)
            blacklist_file.seek(0)
            with mock.patch('six.moves.builtins.open',
                            return_value=blacklist_file):
                result = os_testr.construct_regex('fake_path', None,
                                                  'fake_regex', False)

                expected_regex = (
                    "^(?!fake_regex_3|fake_regex_2|fake_regex_1|"
                    "fake_regex_0).*(fake_regex).*$")
                self.assertEqual(result, expected_regex)

    @mock.patch.object(os_testr, 'print_skips')
    def test_blacklist_regex_with_comment_print_skips(self, print_mock):
        with io.StringIO() as blacklist_file:
            for i in range(4):
                blacklist_file.write(u'fake_regex_%s # Comment\n' % i)
            blacklist_file.seek(0)
            with mock.patch('six.moves.builtins.open',
                            return_value=blacklist_file):
                result = os_testr.construct_regex('fake_path', None,
                                                  None, True)

            expected_regex = ("^(?!fake_regex_3|fake_regex_2|fake_regex_1|"
                              "fake_regex_0).*().*$")
        self.assertEqual(result, expected_regex)
        calls = print_mock.mock_calls
        self.assertEqual(len(calls), 4)
        args = list(map(lambda x: x[1], calls))
        self.assertIn(('fake_regex_0', 'Comment'), args)
        self.assertIn(('fake_regex_1', 'Comment'), args)
        self.assertIn(('fake_regex_2', 'Comment'), args)
        self.assertIn(('fake_regex_3', 'Comment'), args)

    @mock.patch.object(os_testr, 'print_skips')
    def test_blacklist_regex_without_comment_print_skips(self, print_mock):
        with io.StringIO() as blacklist_file:
            for i in range(4):
                blacklist_file.write(u'fake_regex_%s\n' % i)
            blacklist_file.seek(0)
            with mock.patch('six.moves.builtins.open',
                            return_value=blacklist_file):
                result = os_testr.construct_regex('fake_path', None,
                                                  None, True)

            expected_regex = ("^(?!fake_regex_3|fake_regex_2|"
                              "fake_regex_1|fake_regex_0).*().*$")
        self.assertEqual(result, expected_regex)
        calls = print_mock.mock_calls
        self.assertEqual(len(calls), 4)
        args = list(map(lambda x: x[1], calls))
        self.assertIn(('fake_regex_0', ''), args)
        self.assertIn(('fake_regex_1', ''), args)
        self.assertIn(('fake_regex_2', ''), args)
        self.assertIn(('fake_regex_3', ''), args)

    def test_whitelist_regex_without_comments_and_regex_passthrough(self):
        file_contents = u"""regex_a
regex_b"""
        with io.StringIO() as whitelist_file:
            whitelist_file.write(file_contents)
            whitelist_file.seek(0)
            with mock.patch('six.moves.builtins.open',
                            return_value=whitelist_file):
                result = os_testr.construct_regex(None, 'fake_path',
                                                  None, False)

                expected_regex = '^.*(regex_a|regex_b).*$'
                self.assertEqual(result, expected_regex)

    def test_whitelist_regex_without_comments_with_regex_passthrough(self):
        file_contents = u"""regex_a
regex_b"""
        with io.StringIO() as whitelist_file:
            whitelist_file.write(file_contents)
            whitelist_file.seek(0)
            with mock.patch('six.moves.builtins.open',
                            return_value=whitelist_file):
                result = os_testr.construct_regex(None, 'fake_path',
                                                  'fake_regex', False)

                expected_regex = '^.*(fake_regex|regex_a|regex_b).*$'
                self.assertEqual(result, expected_regex)

    def test_blacklist_whitelist_and_regex_passthrough_at_once(self):
        with io.StringIO() as blacklist_file, io.StringIO() as whitelist_file:
            for i in range(4):
                blacklist_file.write(u'fake_regex_%s\n' % i)
            blacklist_file.seek(0)
            whitelist_file.write(u'regex_a\n')
            whitelist_file.write(u'regex_b\n')
            whitelist_file.seek(0)

            with mock.patch('six.moves.builtins.open',
                            side_effect=[blacklist_file, whitelist_file]):
                result = os_testr.construct_regex('fake_path_1', 'fake_path_2',
                                                  'fake_regex', False)

                expected_regex = (
                    "^(?!fake_regex_3|fake_regex_2|fake_regex_1|"
                    "fake_regex_0).*(fake_regex|regex_a|regex_b).*$")
                self.assertEqual(result, expected_regex)


class TestGetRegexFromListFile(base.TestCase):
    def test_get_regex_from_whitelist_file(self):
        file_contents = u"""regex_a
regex_b"""
        with io.StringIO() as whitelist_file:
            whitelist_file.write(file_contents)
            whitelist_file.seek(0)
            with mock.patch('six.moves.builtins.open',
                            return_value=whitelist_file):
                regex = os_testr.get_regex_from_whitelist_file(
                    '/path/to/not_used')
            self.assertEqual('regex_a|regex_b', regex)

    def test_get_regex_from_blacklist_file(self):
        with io.StringIO() as blacklist_file:
            for i in range(4):
                blacklist_file.write(u'fake_regex_%s\n' % i)
            blacklist_file.seek(0)
            with mock.patch('six.moves.builtins.open',
                            return_value=blacklist_file):
                regex = os_testr.get_regex_from_blacklist_file(
                    '/path/to/not_used')
            self.assertEqual('(?!fake_regex_3|fake_regex_2'
                             '|fake_regex_1|fake_regex_0)', regex)


class TestGetTestList(base.TestCase):
    def test__get_test_list(self):
        test_list = os_testr._get_test_list('test__get_test_list')
        self.assertIn('test__get_test_list', test_list[0])

    def test__get_test_list_regex_is_empty(self):
        test_list = os_testr._get_test_list('')
        self.assertIn('', test_list[0])

    def test__get_test_list_regex_is_none(self):
        test_list = os_testr._get_test_list(None)
        # NOTE(masayukig): We should get all of the tests. So we should have
        # more than one test case.
        self.assertGreater(len(test_list), 1)
        self.assertIn('os_testr.tests.test_regex_builder.'
                      'TestGetTestList.test__get_test_list_regex_is_none',
                      test_list)



# Copyright 2015 Hewlett-Packard Development Company, L.P.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import shutil
import subprocess
import tempfile

import testtools

from os_testr.tests import base
from six import StringIO

DEVNULL = open(os.devnull, 'wb')


class TestReturnCodes(base.TestCase):
    def setUp(self):
        super(TestReturnCodes, self).setUp()
        # Setup test dirs
        self.directory = tempfile.mkdtemp(prefix='ostestr-unit')
        self.addCleanup(shutil.rmtree, self.directory)
        self.test_dir = os.path.join(self.directory, 'tests')
        os.mkdir(self.test_dir)
        # Setup Test files
        self.testr_conf_file = os.path.join(self.directory, '.stestr.conf')
        self.setup_cfg_file = os.path.join(self.directory, 'setup.cfg')
        self.passing_file = os.path.join(self.test_dir, 'test_passing.py')
        self.failing_file = os.path.join(self.test_dir, 'test_failing.py')
        self.init_file = os.path.join(self.test_dir, '__init__.py')
        self.setup_py = os.path.join(self.directory, 'setup.py')
        shutil.copy('os_testr/tests/files/stestr-conf', self.testr_conf_file)
        shutil.copy('os_testr/tests/files/passing-tests', self.passing_file)
        shutil.copy('os_testr/tests/files/failing-tests', self.failing_file)
        shutil.copy('setup.py', self.setup_py)
        shutil.copy('os_testr/tests/files/setup.cfg', self.setup_cfg_file)
        shutil.copy('os_testr/tests/files/__init__.py', self.init_file)

        self.stdout = StringIO()
        self.stderr = StringIO()
        # Change directory, run wrapper and check result
        self.addCleanup(os.chdir, os.path.abspath(os.curdir))
        os.chdir(self.directory)

    def assertRunExit(self, cmd, expected, subunit=False):
        p = subprocess.Popen(
            "%s" % cmd, shell=True,
            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        out, err = p.communicate()

        if not subunit:
            self.assertEqual(
                p.returncode, expected,
                "Stdout: %s; Stderr: %s" % (out, err))
        else:
            self.assertEqual(p.returncode, expected,
                             "Expected return code: %s doesn't match actual "
                             "return code of: %s" % (expected, p.returncode))

    def test_default_passing(self):
        self.assertRunExit('ostestr --regex passing', 0)

    def test_default_fails(self):
        self.assertRunExit('ostestr', 1)

    def test_default_passing_no_slowest(self):
        self.assertRunExit('ostestr --no-slowest --regex passing', 0)

    def test_default_fails_no_slowest(self):
        self.assertRunExit('ostestr --no-slowest', 1)

    def test_default_serial_passing(self):
        self.assertRunExit('ostestr --serial --regex passing', 0)

    def test_default_serial_fails(self):
        self.assertRunExit('ostestr --serial', 1)

    def test_testr_subunit_passing(self):
        self.assertRunExit('ostestr --no-pretty --subunit --regex passing', 0,
                           subunit=True)

    @testtools.skip('Skipped because of testrepository lp bug #1411804')
    def test_testr_subunit_fails(self):
        self.assertRunExit('ostestr --no-pretty --subunit', 1, subunit=True)

    def test_testr_no_pretty_passing(self):
        self.assertRunExit('ostestr --no-pretty --regex passing', 0)

    def test_testr_no_pretty_fails(self):
        self.assertRunExit('ostestr --no-pretty', 1)

    def test_list(self):
        self.assertRunExit('ostestr --list', 0)

    def test_no_test(self):
        self.assertRunExit('ostestr --regex a --black-regex a', 1)



# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import re
from unittest import mock

import six

from os_testr import testlist_builder as list_builder
from os_testr.tests import base


class TestBlackReader(base.TestCase):
    def test_black_reader(self):
        blacklist_file = six.StringIO()
        for i in range(4):
            blacklist_file.write('fake_regex_%s\n' % i)
            blacklist_file.write('fake_regex_with_note_%s # note\n' % i)
        blacklist_file.seek(0)
        with mock.patch('six.moves.builtins.open',
                        return_value=blacklist_file):
            result = list_builder.black_reader('fake_path')
        self.assertEqual(2 * 4, len(result))
        note_cnt = 0
        # not assuming ordering, mainly just testing the type
        for r in result:
            self.assertEqual(r[2], [])
            if r[1] == 'note':
                note_cnt += 1
            self.assertIn('search', dir(r[0]))  # like a compiled regex
        self.assertEqual(note_cnt, 4)


class TestConstructList(base.TestCase):
    def test_simple_re(self):
        test_lists = ['fake_test(scen)[tag,bar])', 'fake_test(scen)[egg,foo])']
        with mock.patch('os_testr.regex_builder._get_test_list',
                        return_value=test_lists):
            result = list_builder.construct_list(None,
                                                 None,
                                                 'foo',
                                                 None,
                                                 False)
        self.assertEqual(list(result), ['fake_test(scen)[egg,foo])'])

    def test_simple_black_re(self):
        test_lists = ['fake_test(scen)[tag,bar])', 'fake_test(scen)[egg,foo])']
        with mock.patch('os_testr.regex_builder._get_test_list',
                        return_value=test_lists):
            result = list_builder.construct_list(None,
                                                 None,
                                                 None,
                                                 'foo',
                                                 False)
        self.assertEqual(list(result), ['fake_test(scen)[tag,bar])'])

    def test_blacklist(self):
        black_list = [(re.compile('foo'), 'foo not liked', [])]
        test_lists = ['fake_test(scen)[tag,bar])', 'fake_test(scen)[egg,foo])']
        with mock.patch('os_testr.regex_builder._get_test_list',
                        return_value=test_lists):
            with mock.patch('os_testr.testlist_builder.black_reader',
                            return_value=black_list):
                result = list_builder.construct_list('file',
                                                     None,
                                                     'fake_test',
                                                     None,
                                                     False)
        self.assertEqual(list(result), ['fake_test(scen)[tag,bar])'])

    def test_whitelist(self):
        white_list = 'fake_test1|fake_test2'
        test_lists = ['fake_test1[tg]', 'fake_test2[tg]', 'fake_test3[tg]']
        white_getter = 'os_testr.regex_builder.get_regex_from_whitelist_file'
        with mock.patch('os_testr.regex_builder._get_test_list',
                        return_value=test_lists):
            with mock.patch(white_getter,
                            return_value=white_list):
                result = list_builder.construct_list(None,
                                                     'file',
                                                     None,
                                                     None,
                                                     False)
        self.assertEqual(set(result),
                         set(('fake_test1[tg]', 'fake_test2[tg]')))

    def test_whitelist_blacklist_re(self):
        white_list = 'fake_test1|fake_test2'
        test_lists = ['fake_test1[tg]', 'fake_test2[spam]',
                      'fake_test3[tg,foo]', 'fake_test4[spam]']
        black_list = [(re.compile('spam'), 'spam not liked', [])]
        white_getter = 'os_testr.regex_builder.get_regex_from_whitelist_file'
        with mock.patch('os_testr.regex_builder._get_test_list',
                        return_value=test_lists):
            with mock.patch(white_getter,
                            return_value=white_list):
                with mock.patch('os_testr.testlist_builder.black_reader',
                                return_value=black_list):
                    result = list_builder.construct_list('black_file',
                                                         'white_file',
                                                         'foo',
                                                         None,
                                                         False)
        self.assertEqual(set(result),
                         set(('fake_test1[tg]', 'fake_test3[tg,foo]')))

    def test_overlapping_black_regex(self):

        black_list = [(re.compile('compute.test_keypairs.KeypairsTestV210'),
                       '', []),
                      (re.compile('compute.test_keypairs.KeypairsTestV21'),
                       '', [])]
        test_lists = [
            'compute.test_keypairs.KeypairsTestV210.test_create_keypair',
            'compute.test_keypairs.KeypairsTestV21.test_create_keypair',
            'compute.test_fake.FakeTest.test_fake_test']
        with mock.patch('os_testr.regex_builder._get_test_list',
                        return_value=test_lists):
            with mock.patch('os_testr.testlist_builder.black_reader',
                            return_value=black_list):
                result = list_builder.construct_list('file',
                                                     None,
                                                     'fake_test',
                                                     None,
                                                     False)
        self.assertEqual(
            list(result), ['compute.test_fake.FakeTest.test_fake_test'])



# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_os_testr
----------------------------------

Tests for `os_testr` module.
"""
import io
from unittest import mock

from os_testr import ostestr as os_testr
from os_testr.tests import base


class TestGetParser(base.TestCase):
    def test_pretty(self):
        namespace = os_testr.get_parser(['--pretty'])
        self.assertEqual(True, namespace[0].pretty)
        namespace = os_testr.get_parser(['--no-pretty'])
        self.assertEqual(False, namespace[0].pretty)
        self.assertRaises(SystemExit, os_testr.get_parser,
                          ['--no-pretty', '--pretty'])

    def test_slowest(self):
        namespace = os_testr.get_parser(['--slowest'])
        self.assertEqual(True, namespace[0].slowest)
        namespace = os_testr.get_parser(['--no-slowest'])
        self.assertEqual(False, namespace[0].slowest)
        self.assertRaises(SystemExit, os_testr.get_parser,
                          ['--no-slowest', '--slowest'])

    def test_parallel(self):
        namespace = os_testr.get_parser(['--parallel'])
        self.assertEqual(True, namespace[0].parallel)
        namespace = os_testr.get_parser(['--serial'])
        self.assertEqual(False, namespace[0].parallel)
        self.assertRaises(SystemExit, os_testr.get_parser,
                          ['--parallel', '--serial'])


class TestCallers(base.TestCase):
    def test_no_discover(self):
        namespace = os_testr.get_parser(['-n', 'project.tests.foo'])

        def _fake_exit(arg):
            self.assertTrue(arg)

        def _fake_run(*args, **kwargs):
            return 'project.tests.foo' in args

        with mock.patch.object(os_testr, 'exit', side_effect=_fake_exit), \
                mock.patch.object(os_testr,
                                  'get_parser',
                                  return_value=namespace), \
                mock.patch.object(os_testr,
                                  'call_subunit_run',
                                  side_effect=_fake_run):
            os_testr.main()

    def test_no_discover_path(self):
        namespace = os_testr.get_parser(['-n', 'project/tests/foo'])

        def _fake_exit(arg):
            self.assertTrue(arg)

        def _fake_run(*args, **kwargs):
            return 'project.tests.foo' in args

        with mock.patch.object(os_testr, 'exit', side_effect=_fake_exit), \
                mock.patch.object(os_testr,
                                  'get_parser',
                                  return_value=namespace), \
                mock.patch.object(os_testr,
                                  'call_subunit_run',
                                  side_effect=_fake_run):
            os_testr.main()

    def test_pdb(self):
        namespace = os_testr.get_parser(['--pdb', 'project.tests.foo'])

        def _fake_exit(arg):
            self.assertTrue(arg)

        def _fake_run(*args, **kwargs):
            return 'project.tests.foo' in args

        with mock.patch.object(os_testr, 'exit', side_effect=_fake_exit), \
                mock.patch.object(os_testr,
                                  'get_parser',
                                  return_value=namespace), \
                mock.patch.object(os_testr,
                                  'call_subunit_run',
                                  side_effect=_fake_run):
            os_testr.main()

    def test_pdb_path(self):
        namespace = os_testr.get_parser(['--pdb', 'project/tests/foo'])

        def _fake_exit(arg):
            self.assertTrue(arg)

        def _fake_run(*args, **kwargs):
            return 'project.tests.foo' in args

        with mock.patch.object(os_testr, 'exit', side_effect=_fake_exit), \
                mock.patch.object(os_testr,
                                  'get_parser',
                                  return_value=namespace), \
                mock.patch.object(os_testr,
                                  'call_subunit_run',
                                  side_effect=_fake_run):
            os_testr.main()

    def test_call_subunit_run_pretty(self):
        '''Test call_subunit_run

        Test ostestr call_subunit_run function when:
        Pretty is True
        '''
        pretty = True
        subunit = False

        with mock.patch('subprocess.Popen', autospec=True) as mock_popen:
            mock_popen.return_value.returncode = 0
            mock_popen.return_value.stdout = io.BytesIO()

            os_testr.call_subunit_run('project.tests.foo', pretty, subunit)

            # Validate Popen was called three times
            self.assertTrue(mock_popen.called, 'Popen was never called')
            count = mock_popen.call_count
            self.assertEqual(3, count, 'Popen was called %s'
                             ' instead of 3 times' % count)

            # Validate Popen called the right functions
            called = mock_popen.call_args_list
            msg = "Function %s not called"
            function = ['python', '-m', 'subunit.run', 'project.tests.foo']
            self.assertIn(function, called[0][0], msg % 'subunit.run')
            function = ['stestr', 'load', '--subunit']
            self.assertIn(function, called[1][0], msg % 'testr load')
            function = ['subunit-trace', '--no-failure-debug', '-f']
            self.assertIn(function, called[2][0], msg % 'subunit-trace')

    def test_call_subunit_run_sub(self):
        '''Test call_subunit run

        Test ostestr call_subunit_run function when:
        Pretty is False and Subunit is True
        '''
        pretty = False
        subunit = True

        with mock.patch('subprocess.Popen', autospec=True) as mock_popen:
            os_testr.call_subunit_run('project.tests.foo', pretty, subunit)

            # Validate Popen was called once
            self.assertTrue(mock_popen.called, 'Popen was never called')
            count = mock_popen.call_count
            self.assertEqual(1, count, 'Popen was called more than once')

            # Validate Popen called the right function
            called = mock_popen.call_args
            function = ['stestr', 'load', '--subunit']
            self.assertIn(function, called[0], "testr load not called")

    def test_call_subunit_run_testtools(self):
        '''Test call_subunit_run

        Test ostestr call_subunit_run function when:
        Pretty is False and Subunit is False
        '''
        pretty = False
        subunit = False

        with mock.patch('testtools.run.main', autospec=True) as mock_run:
            os_testr.call_subunit_run('project.tests.foo', pretty, subunit)

            # Validate testtool.run was called once
            self.assertTrue(mock_run.called, 'testtools.run was never called')
            count = mock_run.call_count
            self.assertEqual(1, count, 'testtools.run called more than once')

    def test_parse_legacy_testrconf_discover(self):
        '''Test _parse_testrconf

        Test ostestr _parse_testrconf function when:
        -t is not specified and discover is specified
        '''
        testrconf_data = u"""
[DEFAULT]
test_command=OS_STDOUT_CAPTURE=${OS_STDOUT_CAPTURE:-1} \
             OS_STDERR_CAPTURE=${OS_STDERR_CAPTURE:-1} \
             OS_TEST_TIMEOUT=${OS_TEST_TIMEOUT:-60} \
             ${PYTHON:-python} -m subunit.run discover mytestdir \
             $LISTOPT $IDOPTION
test_id_option=--load-list $IDFILE
test_list_option=--list
group_regex=([^\\.]+\\.)+
"""
        with io.StringIO() as testrconf_data_file:
            testrconf_data_file.write(testrconf_data)
            testrconf_data_file.seek(0)

            with mock.patch('six.moves.builtins.open',
                            return_value=testrconf_data_file, autospec=True):
                parsed_values = os_testr._parse_testrconf()
                # validate the discovery of the options from the legacy
                # .testr.conf
                self.assertEqual(parsed_values, ('mytestdir', None,
                                                 r'([^\.]+\.)+'))

    def test_parse_legacy_testrconf_topdir(self):
        '''Test parse_testrconf

        Test ostestr _parse_testrconf function when:
        -t is specified
        '''
        testrconf_data = u"""
[DEFAULT]
test_command=OS_STDOUT_CAPTURE=${OS_STDOUT_CAPTURE:-1} \
             OS_STDERR_CAPTURE=${OS_STDERR_CAPTURE:-1} \
             OS_TEST_TIMEOUT=${OS_TEST_TIMEOUT:-60} \
             ${PYTHON:-python} -m subunit.run discover -t .. mytestdir \
             $LISTOPT $IDOPTION
test_id_option=--load-list $IDFILE
test_list_option=--list
group_regex=([^\\.]+\\.)+
"""
        with io.StringIO() as testrconf_data_file:
            testrconf_data_file.write(testrconf_data)
            testrconf_data_file.seek(0)

            with mock.patch('six.moves.builtins.open',
                            return_value=testrconf_data_file, autospec=True):
                parsed_values = os_testr._parse_testrconf()
                # validate the discovery of the options from the legacy
                # .testr.conf
                self.assertEqual(parsed_values, ('mytestdir', '..',
                                                 r'([^\.]+\.)+'))



#!/usr/bin/env python3
# Copyright 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import argparse
import copy
import io
import os
import subprocess
import sys
import warnings

import pbr.version
import six.moves

from stestr import commands
from subunit import run as subunit_run
from testtools import run as testtools_run

from os_testr import regex_builder as rb


__version__ = pbr.version.VersionInfo('os_testr').version_string()


def get_parser(args):
    parser = argparse.ArgumentParser(
        description='Tool to run openstack tests')
    parser.add_argument('--version', action='version',
                        version='%s' % __version__)
    parser.add_argument('--blacklist-file', '-b', '--blacklist_file',
                        help='Path to a blacklist file, this file '
                             'contains a separate regex exclude on each '
                             'newline')
    parser.add_argument('--whitelist-file', '-w', '--whitelist_file',
                        help='Path to a whitelist file, this file '
                             'contains a separate regex on each newline.')
    group = parser.add_mutually_exclusive_group()
    group.add_argument('--regex', '-r',
                       help='A normal testr selection regex.')
    group.add_argument('--path', metavar='FILE_OR_DIRECTORY',
                       help='A file name or directory of tests to run.')
    group.add_argument('--no-discover', '-n', metavar='TEST_ID',
                       help="Takes in a single test to bypasses test "
                            "discover and just execute the test specified. "
                            "A file name may be used in place of a test "
                            "name.")
    parser.add_argument('--black-regex', '-B',
                        help='Test rejection regex. If a test cases name '
                        'matches on re.search() operation , '
                        'it will be removed from the final test list. '
                        'Effectively the black-regex is added to '
                        ' black regex list, but you do need to edit a file. '
                        'The black filtering happens after the initial '
                        ' white selection, which by default is everything.')
    pretty = parser.add_mutually_exclusive_group()
    pretty.add_argument('--pretty', '-p', dest='pretty', action='store_true',
                        help='Print pretty output from subunit-trace. This is '
                             'mutually exclusive with --subunit')
    pretty.add_argument('--no-pretty', dest='pretty', action='store_false',
                        help='Disable the pretty output with subunit-trace')
    parser.add_argument('--subunit', '-s', action='store_true',
                        help='output the raw subunit v2 from the test run '
                             'this is mutually exclusive with --pretty')
    parser.add_argument('--list', '-l', action='store_true',
                        help='List all the tests which will be run.')
    parser.add_argument('--color', action='store_true',
                        help='Use color in the pretty output')
    slowest = parser.add_mutually_exclusive_group()
    slowest.add_argument('--slowest', dest='slowest', action='store_true',
                         help="after the test run print the slowest tests")
    slowest.add_argument('--no-slowest', dest='slowest', action='store_false',
                         help="after the test run don't print the slowest "
                              "tests")
    parser.add_argument('--pdb', metavar='TEST_ID',
                        help='Run a single test that has pdb traces added')
    parallel = parser.add_mutually_exclusive_group()
    parallel.add_argument('--parallel', dest='parallel', action='store_true',
                          help='Run tests in parallel (this is the default)')
    parallel.add_argument('--serial', dest='parallel', action='store_false',
                          help='Run tests serially')
    parser.add_argument('--concurrency', '-c', type=int, metavar='WORKERS',
                        default=0,
                        help='The number of workers to use when running in '
                             'parallel. By default this is the number of cpus')
    parser.add_argument('--until-failure', action='store_true',
                        help='Run the tests in a loop until a failure is '
                             'encountered. Running with subunit or pretty'
                             'output enable will force the loop to run tests'
                             'serially')
    parser.add_argument('--print-exclude', action='store_true',
                        help='If an exclude file is used this option will '
                             'prints the comment from the same line and all '
                             'skipped tests before the test run')
    parser.set_defaults(pretty=True, slowest=True, parallel=True)
    return parser.parse_known_args(args)


def _parse_testrconf():
    # Parse the legacy .testr.conf file.
    test_dir = None
    top_dir = None
    group_regex = None

    with open('.testr.conf', 'r') as testr_conf_file:
        config = six.moves.configparser.ConfigParser()
        config.readfp(testr_conf_file)
        test_command = config.get('DEFAULT', 'test_command')
        group_regex = None
        if config.has_option('DEFAULT', 'group_regex'):
            group_regex = config.get('DEFAULT', 'group_regex')

    for line in test_command.split('\n'):
        if 'subunit.run discover' in line:
            command_parts = line.split(' ')
            top_dir_present = '-t' in line
            for idx, val in enumerate(command_parts):
                if top_dir_present:
                    if val == '-t':
                        top_dir = command_parts[idx + 1]
                        test_dir = command_parts[idx + 2]
                else:
                    if val == 'discover':
                        test_dir = command_parts[idx + 1]
    return (test_dir, top_dir, group_regex)


def call_testr(regex, subunit, pretty, list_tests, slowest, parallel, concur,
               until_failure, color, others=None, blacklist_file=None,
               whitelist_file=None, black_regex=None, load_list=None):
    # Handle missing .stestr.conf from users from before stestr migration
    test_dir = None
    top_dir = None
    group_regex = None
    if not os.path.isfile('.stestr.conf') and os.path.isfile('.testr.conf'):
        msg = ('No .stestr.conf file found in the CWD. Please create one to '
               'replace the .testr.conf file. You can find a script to do '
               'this in the stestr git repository.')
        warnings.warn(msg)

        test_dir, top_dir, group_regex = _parse_testrconf()
    elif not os.path.isfile(
        '.testr.conf') and not os.path.isfile('.stestr.conf'):
        msg = ('No .stestr.conf found, please create one.')
        print(msg)
        sys.exit(1)

    regexes = None
    if regex:
        regexes = regex.split()
    serial = not parallel
    if list_tests:
        # TODO(mtreinish): remove init call after list command detects and
        # autocreates the repository
        if not os.path.isdir('.stestr'):
            commands.init_command()
        return commands.list_command(filters=regexes)
    return_code = commands.run_command(filters=regexes, subunit_out=subunit,
                                       concurrency=concur, test_path=test_dir,
                                       top_dir=top_dir,
                                       group_regex=group_regex,
                                       until_failure=until_failure,
                                       serial=serial, pretty_out=pretty,
                                       load_list=load_list,
                                       blacklist_file=blacklist_file,
                                       whitelist_file=whitelist_file,
                                       black_regex=black_regex)

    if slowest:
        sys.stdout.write("\nSlowest Tests:\n")
        commands.slowest_command()
    return return_code


def call_subunit_run(test_id, pretty, subunit):
    env = copy.deepcopy(os.environ)
    cmd_save_results = ['stestr', 'load', '--subunit']
    if not os.path.isdir('.stestr'):
        commands.init_command()

    if pretty:
        # Use subunit run module
        cmd = ['python', '-m', 'subunit.run', test_id]
        ps = subprocess.Popen(cmd, env=env, stdout=subprocess.PIPE)
        # Save subunit results via testr
        pfile = subprocess.Popen(cmd_save_results, env=env,
                                 stdin=ps.stdout, stdout=subprocess.PIPE)
        ps.stdout.close()
        # Transform output via subunit-trace
        proc = subprocess.Popen(['subunit-trace', '--no-failure-debug', '-f'],
                                env=env, stdin=pfile.stdout)
        pfile.stdout.close()
        proc.communicate()
        return proc.returncode
    elif subunit:
        sstdout = io.BytesIO()
        subunit_run.main([sys.argv[0], test_id], sstdout)
        pfile = subprocess.Popen(cmd_save_results, env=env,
                                 stdin=subprocess.PIPE)
        pfile.communicate(input=sstdout.getvalue())
    else:
        testtools_run.main([sys.argv[0], test_id], sys.stdout)


def _select_and_call_runner(opts, exclude_regex, others):
    ec = 1

    if not opts.no_discover and not opts.pdb:
        ec = call_testr(exclude_regex, opts.subunit, opts.pretty, opts.list,
                        opts.slowest, opts.parallel, opts.concurrency,
                        opts.until_failure, opts.color, others,
                        blacklist_file=opts.blacklist_file,
                        whitelist_file=opts.whitelist_file,
                        black_regex=opts.black_regex)
    else:
        if others:
            print('Unexpected arguments: ' + ' '.join(others))
            return 2
        test_to_run = opts.no_discover or opts.pdb
        if test_to_run.find('/') != -1:
            test_to_run = rb.path_to_regex(test_to_run)
        ec = call_subunit_run(test_to_run, opts.pretty, opts.subunit)
    return ec


def ostestr(args):
    msg = ('Deprecate: ostestr command is deprecated now. Use stestr '
           'command instead. For more information: '
           'https://docs.openstack.org/os-testr/latest/user/ostestr.html')
    warnings.warn(msg)
    opts, others = get_parser(args)
    if opts.pretty and opts.subunit:
        msg = ('Subunit output and pretty output cannot be specified at the '
               'same time')
        print(msg)
        return 2
    if opts.list and opts.no_discover:
        msg = ('you can not list tests when you are bypassing discovery to '
               'run a single test')
        print(msg)
        return 3
    if not opts.parallel and opts.concurrency:
        msg = "You can't specify a concurrency to use when running serially"
        print(msg)
        return 4
    if (opts.pdb or opts.no_discover) and opts.until_failure:
        msg = "You can not use until_failure mode with pdb or no-discover"
        print(msg)
        return 5
    if ((opts.pdb or opts.no_discover) and
            (opts.blacklist_file or opts.whitelist_file)):
        msg = "You can not use blacklist or whitelist with pdb or no-discover"
        print(msg)
        return 6
    if ((opts.pdb or opts.no_discover) and (opts.black_regex)):
        msg = "You can not use black-regex with pdb or no-discover"
        print(msg)
        return 7

    if opts.path:
        regex = rb.path_to_regex(opts.path)
    else:
        regex = opts.regex

    return _select_and_call_runner(opts, regex, others)


def main():
    exit(ostestr(sys.argv[1:]))


if __name__ == '__main__':
    main()
